print("import: 'pyreadline'")
import pyreadline

print("import: 'pyreadline.clipboard'")
import pyreadline.clipboard

print("import: 'pyreadline.console'")
import pyreadline.console

print("import: 'pyreadline.keysyms'")
import pyreadline.keysyms

print("import: 'pyreadline.lineeditor'")
import pyreadline.lineeditor

print("import: 'pyreadline.modes'")
import pyreadline.modes

